/*
 * File name: gnuplot.h
 * Date:      2010/06/27
 * Author:    Miroslav Kulich
 */

#ifndef IMR_GNUPLOT
#define IMR_GNUPLOT

#include <stdio.h>
#include <string>
#include <vector>
#include <iostream>
#include <fstream>

#include <sstream>              // for std::ostringstream
#include <stdexcept>
#include <cstdio>
#include <cstdlib>              // for getenv()

class Gnuplot {

  public:
    static const int endl;
    Gnuplot();
    ~Gnuplot();
    template <class T>
      Gnuplot& operator<<(const T &cmdstr);

//    template <const int>
//      Gnuplot& operator<<(const int x);

//    template<const int T>
     Gnuplot& operator<< (const int x);
     Gnuplot& operator<< (const double x);
      
      
    Gnuplot& operator<<(const std::string &cmdstr);
    Gnuplot& operator<<(const char cmdstr[]);
    template<class T>  Gnuplot& operator<<(const std::vector<T>);
    template<class T>  Gnuplot& operator<<(const std::vector<std::vector<T> >);
    
    Gnuplot& showonscreen();
    Gnuplot& cmd(const std::string &cmdstr);

  private:
    FILE *gnucmd;
    
};


    
    Gnuplot::Gnuplot() {
      gnucmd = popen("gnuplot","w");

    }
    
    Gnuplot::~Gnuplot() {
      pclose(gnucmd);
    }

    Gnuplot& Gnuplot::showonscreen() {
     *this << "set output" << Gnuplot::endl;
     *this << "set terminal x11" << Gnuplot::endl;
     return *this;
  };

  Gnuplot& Gnuplot::cmd(const std::string &cmdstr) {
//   if( !(valid) ) {
//         return *this;
//   }

//    fputs( (cmdstr+"\n").c_str(), gnucmd );
    fputs( (cmdstr).c_str(), gnucmd );
    fflush(gnucmd);
    return *this;
  }


    template <class T>
      Gnuplot& Gnuplot::operator<<(const T &cmdstr) {
      puts("ERROR operator <<");
      return *this;
     }

//    template<const int>
     Gnuplot& Gnuplot::operator<<(const int x){
      if  (x == Gnuplot::endl) {
          cmd("\n");
      } else {
        std::stringstream out;
        out << x;
        cmd(out.str().c_str());
      }
      return *this;
    }

     Gnuplot& Gnuplot::operator<<(const double x){
        std::stringstream out;
        out << x;
        cmd(out.str().c_str());
      return *this;
    }


    Gnuplot& Gnuplot::operator<<(const std::string &cmdstr){
      cmd(cmdstr);
     return(*this);
    }

    Gnuplot& Gnuplot::operator<<(const char cmdstr[]){
      cmd(cmdstr);
     return(*this);
    }

    template<class T>
      Gnuplot& Gnuplot::operator<<(const std::vector<T> vect) {
          typename std::vector<T>::const_iterator i;
          for(i=vect.begin();i!=vect.end();i++ ) {
            *this << *i << Gnuplot::endl;
          }
          *this << "e"<< Gnuplot::endl;
        return *this;
      }

    template<class T>
      Gnuplot& Gnuplot::operator<<(const std::vector<std::vector<T> > vect) {
          typename std::vector<std::vector<T> >::const_iterator i;
          typename std::vector<T>::const_iterator j;
          
          for(i=vect.begin();i!=vect.end();i++ ) {
            for(j=i->begin();j!=i->end();j++ ) {
              *this << *j << " ";
            }
            *this << Gnuplot::endl;
          }
          *this << Gnuplot::endl;
          *this << "e" << Gnuplot::endl;
        return *this;
      }



const int Gnuplot::endl = -999999;

#endif

/* end of gnuplot.h */
